define(["require", "exports", "./jsbn_base64", "./JSEncryptRSAKey"], function (require, exports, jsbn_base64_1, JSEncryptRSAKey_1) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    exports.JSEncrypt = void 0;
    var JSEncrypt = (function () {
        function JSEncrypt(options) {
            options = options || {};
            this.default_key_size = options.default_key_size ? parseInt(options.default_key_size, 10) : 1024;
            this.default_public_exponent = options.default_public_exponent || "010001";
            this.log = options.log || false;
            this.key = null;
        }
        JSEncrypt.prototype.setKey = function (key) {
            if (this.log && this.key) {
                console.warn("A key was already set, overriding existing.");
            }
            this.key = new JSEncryptRSAKey_1.JSEncryptRSAKey(key);
        };
        JSEncrypt.prototype.setPrivateKey = function (privkey) {
            this.setKey(privkey);
        };
        JSEncrypt.prototype.setPublicKey = function (pubkey) {
            this.setKey(pubkey);
        };
        JSEncrypt.prototype.decrypt = function (str) {
            try {
                return this.getKey().decrypt(jsbn_base64_1.b64tohex(str));
            }
            catch (ex) {
                return false;
            }
        };
        JSEncrypt.prototype.encrypt = function (str) {
            try {
                return jsbn_base64_1.hex2b64(this.getKey().encrypt(str));
            }
            catch (ex) {
                return false;
            }
        };
        JSEncrypt.prototype.sign = function (str, digestMethod, digestName) {
            try {
                return jsbn_base64_1.hex2b64(this.getKey().sign(str, digestMethod, digestName));
            }
            catch (ex) {
                return false;
            }
        };
        JSEncrypt.prototype.verify = function (str, signature, digestMethod) {
            try {
                return this.getKey().verify(str, jsbn_base64_1.b64tohex(signature), digestMethod);
            }
            catch (ex) {
                return false;
            }
        };
        JSEncrypt.prototype.getKey = function (cb) {
            if (!this.key) {
                this.key = new JSEncryptRSAKey_1.JSEncryptRSAKey();
                if (cb && {}.toString.call(cb) === "[object Function]") {
                    this.key.generateAsync(this.default_key_size, this.default_public_exponent, cb);
                    return;
                }
                this.key.generate(this.default_key_size, this.default_public_exponent);
            }
            return this.key;
        };
        return JSEncrypt;
    }());
    exports.JSEncrypt = JSEncrypt;
});
